import time
from datetime import datetime
from flask import render_template
import os, time, traceback
from app import app


if not os.path.isfile("app.log"):
    with open("app.log", "w+") as a:
        pass


class Logger:
    def __init__(self, logfile):
        self.logfile = logfile
        if not os.path.isfile(logfile):
            with open(logfile, "w+") as a:
                pass

    def log(self, logstring, end="\n"):
        print(logstring)
        with open(self.logfile, "a") as f:
            f.write(logstring + end)

    def get_log(self):
        with open(self.logfile, "r") as f:
            return f.read()

    def get_log_lines(self):
        with open(self.logfile, "r") as f:
            return f.readlines()


logger = Logger("app.log")


def getHTMLErrorString(title, error):
    return f"<p class='errorheader'>{title}</p><p class='errortext'>{error}</p>"


def error_handler(method):
    def apply(*args, **kw):
        try:
            t = time.ctime()
            name = method.__name__
            logger.log(f"Method '{name}' at {t}")
            start = time.time()
            result = method(*args, **kw)
            end = time.time()
            logger.log(
                "{} took {} ms".format(
                    method.__name__, "%2.2f" % ((end - start) * 1000)
                )
            )
            return result
        except Exception as e:
            title = f"Error running {method.__name__} with args {args} and kwargs {kw}"
            string = traceback.format_exc()
            logger.log(title + "\n" + string)
            error = getHTMLErrorString(title, string.replace("\n", "<br/>"))
            from .routes import errorblog

            errorblog.manager.new_post(
                0,
                datetime.strftime(
                    datetime.utcfromtimestamp(int(float(time.time()))),
                    "%m-%d-%y %H:%M:%S",
                ),
                title,
                string,
            )
            return render_template(
                "user/error.html",
                error="An error occured and has been reported. Sorry for the inconvenience",
            )
            # return render_template("user/error.html", error=error)

    apply.__dict__["mname"] = method.__dict__.get("mname") or method.__name__
    return apply


def error_no_redirect(method):
    def apply(*args, **kw):
        try:
            t = time.ctime()
            name = method.__name__
            logger.log(f"Method '{name}' at {t}")
            start = time.time()
            result = method(*args, **kw)
            end = time.time()
            logger.log(
                "{} took {} ms".format(
                    method.__name__, "%2.2f" % ((end - start) * 1000)
                )
            )
            return result
        except Exception as e:
            title = f"Error running {method.__name__} with args {args} and kwargs {kw}"
            string = traceback.format_exc()
            logger.log(title + "\n" + string)

    return apply
